%This code estimates a Bayesian VAR for monetary policy analysis using data
%from Chile on three variables: inflation, the output gap, and the monetary
%policy rate (MPR), for the period 2001:Q3-2019:Q3. The Bayesian VAR uses a
%Minnesota prior.
%It identifies monetary policy shocks using sign restrictions: only
%responses to MPR shocks that satisfy the following criteria are accepted:
% 1. The MPR increases above its mean in the quarter the shock hits and the
% following two quarters.
% 2. Inflation decreases below its mean in the two quarters after the
% quarter in which the shock hits. The response of inflation in the quarter
% the shock hits is left unrestricted to allow for the possibility of a
% "prize puzzle" effect.
% 3. The output gap shows similar responses than inflation: it decreases in
% the two quarters after the quarter in which the shock hits.
%The code plots impulse-responses, with error bands computed from the
%distribution of impulse responses generated by the Gibbs sampling
%algorithm.
%If you want to alter the specification (number of variables, number of
%lags, etc.), you have to read and adjust the code line by line. It's not
%written so as to be modified by setting a few options.
%The notation is based on the handbook and accompanying
%code by Andrew Blake and Haroon Mumtaz: Applied Bayesian Econometrics for
%Central Bankers. The code uses functions provided by the authors.
%Juan Guerra-Salas
%November 2020

clear
addpath('functions'); %functions written by Blake and Mumtaz
%load data
addpath('data');
data=xlsread('\data\dataChile.xlsx'); %inflation, output gap, MPR. 2001:Q3-2019:Q3 

%Plot variables
tt=2001.5:0.25:2019.5;
figure(1)
subplot(3,1,1)
plot(tt,data(:,1)); grid on
title('Inflation');
axis tight
subplot(3,1,2)
plot(tt,data(:,2)); grid on
title('Output gap');
axis tight
subplot(3,1,3)
plot(tt,data(:,3)); grid on
title('Monetary policy rate');
axis tight

N=size(data,2); %number of endogenous variables
L=1; %number of lags in the VAR
Y=data; %matrix of endogenous variables
X=[ones(size(Y,1),1) lag0(data,1)]; %matrix of regressors: constant and first lag of variables
Y=Y(L+1:end,:); %adjust sample for observations that are lost due to lags
X=X(L+1:end,:);
T=rows(X); %number of observations (adjusted)

%Inputs and parameters for Bayesian estimation

%Standard deviation of residual from AR(1) regression estimated by OLS, to
%be used in setting the prior
%First variable:
y=Y(:,1); %dependent variable
x=X(:,1:2); %regressors: constant and first lag of the variable
b0=inv(x'*x)*(x'*y); %OLS estimate
s1=sqrt(((y-x*b0)'*(y-x*b0))/(rows(y)-2)); %standard deviation of residual
%Second variable:
y=Y(:,2); %dependent variable
x=X(:,[1 3]); %regressors: constant and first lag of the variable
b0=inv(x'*x)*(x'*y); %OLS estimate
s2=sqrt(((y-x*b0)'*(y-x*b0))/(rows(y)-2)); %standard deviation of residual 
%Third variable
y=Y(:,3); %dependent variable
x=X(:,[1 4]); %regressors: constant and first lag of the variable
b0=inv(x'*x)*(x'*y); %OLS estimate
s3=sqrt(((y-x*b0)'*(y-x*b0))/(rows(y)-2)); %standard deviation of residual

%Parameters to control prior
lambda1=0.5; %tightness of prior on coefficients of a variable's own lags. The higher, the looser. As it tends to 0, post mean tends to prior mean.
lambda2=0.5; %tightness of prior on lags of other variables. As it tends to 0, post mean goes to zero (prior mean). When =1, no distinction between lags of dependent var and other vars.
lambda3=1; %tightness of prior on coeffs of lags higher than 1. As it increases, coeffs on higher lags are shrunk to 0 more tightly.
lambda4=10^5;1;  %tightness of prior on the constant term. The higher the looser.
%Canova (2007, p. 380) reports values "typically used:" lambda1=0.2,
%lambda2=0.5, lamda3=1 or 2, lamda4=10^5.
%Note: lambda1=0.2 is an informative prior in this monetary policy
%application: IRFs deviate from results of classical estimation. 0.5 was
%"loose" enough to be uninformative (~flat).

%Bayesian estimation

%Step 1: set priors and starting values
%Prior mean of the coefficients
B0=zeros((N*L+1),N);
for i=1:N
    B0(i+1,i)=0.95;1;
end
B0=vec(B0);
%Variance of prior on coefficients
H=eye(N*(N*L+1),N*(N*L+1)); 
%First equation
H(1,1)=(s1*lambda4)^2; %prior variance on constant term
H(2,2)=(lambda1)^2; %prior variance on coeff first lag of same variable
H(3,3)=((s1*lambda1*lambda2)/s2)^2; %prior variance on coeff first lag of other variable
H(4,4)=((s1*lambda1*lambda2)/s3)^2; %prior variance on coeff first lag of other variable
%Second equation
H(5,5)=(s2*lambda4)^2;
H(6,6)=((s2*lambda1*lambda2)/s1)^2;
H(7,7)=(lambda1)^2;
H(8,8)=((s2*lambda1*lambda2)/s3)^2;
%Third equation
H(9,9)=(s3*lambda4)^2;
H(10,10)=((s3*lambda1*lambda2)/s1)^2;
H(11,11)=((s3*lambda1*lambda2)/s2)^2;
H(12,12)=(lambda1)^2;

%Prior for variance-covariance matrix of residuals
S=eye(N); %prior scale matrix
alpha=N+1; %prior degrees of freedom
%the prior for the covariance matrix is non-informative, since the prior
%degrees of freedom are the "minimum possible N+1" (Blake and Mumtaz, p.
%34).

%Starting values for the Gibbs sampling algorithm
%Sigma=eye(N);
sigma=eye(N);
betaols=vec(inv(X'*X)*(X'*Y));

REPS=5000; %total number of Gibbs iterations
BURN=3000; %number of burn-in iterations
out=zeros(REPS-BURN,20,N); %array that will store IRFs of all Gibbs iterations
jj=1;
for i=1:REPS %main Gibbs sampling loop

%Step 2: Sample B conditional on sigma from posterior N(M*,V*)
M=inv(inv(H)+kron(inv(sigma),X'*X))*(inv(H)*B0+kron(inv(sigma),X'*X)*betaols);
V=inv(inv(H)+kron(inv(sigma),X'*X));
%Check for stability of the VAR: only keep draws that generate stable VAR
check=-1;
    while check<0
        beta=M+(randn(1,N*(N*L+1))*chol(V))'; %random draw of coefficients
        CH=stability(beta,N,L); %stability is a function written by Blake and Mumtaz
        if CH==0
            check=10;
        end
    end

%Step 3: Sample sigma conditional on B from posterior IW(scale,T+alpha);    
e=Y-X*reshape(beta,N*L+1,N); %residuals
scale=e'*e+S; %scale matrix
sigma=IWPQ(T+alpha,inv(scale)); %random draw of variance; IWPQ is a function written by Blake and Mumtaz

    if i>=BURN %once we're past the burn-in iterations, compute IRFs and keep those that satisfy sign restrictions
        chck=-1;
        while chck<0
            K=randn(N,N); %random matrix
            Q=getqr(K); %QR decomposition of matrix K
            A0hat=chol(sigma); %Cholesky decomposition of var-cov matrix of residuals
            A0hat1=(Q*A0hat);  %Candidate draw
            %Compute impulse-response functions
            yhat=zeros(20,N); %matrix that will store IRFs of the current Gibbs iteration
            vhat=zeros(20,N); %matrix that will contain the shock
            vhat(L+1,3)=1; %here is the shock to the MPR
            for j=2:20
                yhat(j,:)=[ 0 yhat(j-1,:)]*reshape(beta,N*L+1,N)+vhat(j,:)*A0hat1;
            end
            %Specify sign restrictions
            e1=yhat(3:4,1)<0;  %response of inflation: <0 in the 2 quarters after the quarter in which shock hits (not contemporaneously to allow for prize puzzle)
            e2=yhat(3:4,2)<0;  %response of output gap: <0 in the 2 quarters after the quarter in which shock hits (not contemp for consistency)
            e3=yhat(2:4,3)>0;  %response of MPR: >0 in quarter the shock hits and the 2 quarters after that
            if sum(e1)+sum(e2)+sum(e3)==7 %check if sign restrictions are satisfied; if they are, keep IRFs
                chck=10;
            else
            %check if sign restrictions are satisfied but with reversed sign; if they are, keep IRFs
            e1=-yhat(3:4,1)<0;  %response of inflation
            e2=-yhat(3:4,2)<0;  %response of output gap
            e3=-yhat(2:4,3)>0;  %response of MPR
            if sum(e1)+sum(e2)+sum(e3)==7
                yhat(:,:)=-yhat(:,:);
                chck=10;
            end
            end
        end %identification with sign restrictions ends here
    out(jj,:,:)=yhat; %save IRFs that satisfy sign restrictions
    jj=jj+1;
    end
end

%Normalize IRFs
norm=0; %Set to 1 to obtain normalized IRFs, i.e., shock of size 1
if norm==1
    for j=1:(REPS-BURN)
        out(j,:,:)=out(j,:,:)/out(j,2,3);
    end
end
  
%Plot IRFs with 68% error bands
figure(2)
subplot(3,1,1)
temp=out(:,:,1);
temp1=prctile(temp,[50 16 84],1)';
plot(temp1(2:end,:));
title('Response of inflation');
axis tight
subplot(3,1,2)
temp=out(:,:,2);
temp1=prctile(temp,[50 16 84],1)';
plot(temp1(2:end,:));
title('Response of output gap');
axis tight
subplot(3,1,3)
temp=out(:,:,3);
temp1=prctile(temp,[50 16 84],1)';
plot(temp1(2:end,:));
title('Response of monetary policy rate');
axis tight
legend('Median Response','Lower 16%','Upper 84%');